; DBFLOP = Don't Boot FLOPpy

ofs             equ     offset
cr              =       13
lf              =       10

jmps            macro   lbl
                jmp     short lbl
                endm

code            segment
                assume  cs:code
                org     100h
entry:          jmp     start

prompt          db      cr,lf
db "    Insert a formatted floppy in drive A.  Now, press Enter and the floppy's",cr,lf
db "    boot sector will be rewritten so that attempting to boot the floppy will",cr,lf
db "    result in your fixed disk being booted instead.  Any data on the floppy",cr,lf
db "    will not be altered.  (DOS diskettes only!)  Or PRESS ESCAPE TO EXIT.",cr,lf
db "    $"

norecog         db      cr,lf
db "    Unrecognizeable floppy format...operation aborted.",lf,"$"

nofit           db      cr,lf
db "    There was a problem...the code won't fit in boot sector...sorry.",lf,"$"

nowrite         db      cr,lf
db "    Error writing to floppy boot sector.",lf,"$"

nbsize          = start - newboot       ; size of new boot code:
brsize          = start - bootread      ; size of fixed disk boot read code
isize           = bootread - newboot    ; size of initialization code

newboot:        sub     ax,ax           ; insure initialization of ds and es
                mov     ds,ax
                mov     es,ax
;*******************************************************************************
; Note:  The address 7c00h in the mov si instruction below is incorrect and is
; merely a place holder.  The actual value of this location is computed then
; stored in place of 7c00h by the code that follows.
;*******************************************************************************
movsi:          mov     si,7c00h        ; move our code out of harm's way
                mov     di,7c00h + 200h ; just beyond boot sector buffer
                mov     cx,brsize       ; get size of boot sector read code
                cld
                rep     movsb           ; move the code
                mov     bx,7c00h + 200h
                jmp     bx              ; jump to boot read code
bootread:       mov     ax,201h         ; read fixed disk boot sector
                mov     bx,7c00h        ; into boot location
                mov     cx,1            ; cylender 0, sector 1
                mov     dx,80h          ; of head 0, drive C
                int     13h
                mov     bx,7c00h + 1feh ; check for valid boot sector signature
                cmp     [bx],0aa55h
                jne     loadbasic       ; not valid, load ROM BASIC
                mov     bx,7c00h        ; jump to fixed disk boot code
                jmp     bx
loadbasic:      int     18h

start:          mov     ah,9            ; display (insert disk) prompt
                mov     dx,ofs prompt
                int     21h
readkey:        mov     ah,0            ; get keypress
                int     16h
                cmp     ax,1c0dh        ; Enter key?
                je      doit            ; yes, rewrite floppy boot code
                cmp     ax,11bh         ; Escape key?
                je      abort           ; no, return to DOS
                jmps    readkey         ; get another key
abort:          mov     al,1            ; terminate with exit code 1
                jmps    exit

doit:           mov     ax,201h         ; read 1 sector
                mov     bx,ofs bootsec  ; into our buffer
                mov     cx,1            ; cylender 0, sector 1
                sub     dx,dx           ; of head 0, drive 0 (A drive)
                int     13h

                mov     si,ofs bootsec
                lodsw                   ; get first word
                cmp     al,0ebh         ; is it a jmp instruction?
                je      got_jmp         ; yes, continue
                mov     ah,9            ; not a recognized format
                mov     dx,ofs norecog  ; thus, say so and exit
                int     21h
                mov     al,2            ; terminate with exit code of 2
                jmps    exit

got_jmp:        mov     al,ah           ; put jump location to di
                sub     ah,ah
                mov     bx,ax           ; save for later
                mov     di,si           ; (si was advanced by 2 with lodsw)
                add     di,ax           ; so di now points to current boot code
                mov     ax,di           ; make sure that the new code will fit
                add     ax,nbsize
                sub     ax,ofs bootsec
                cmp     ax,512          ; overflow one sector?
                jbe     itsok           ; no, it's ok to press on
                mov     ah,9            ; yes, it would, complain
                mov     dx,ofs nofit    ; tell 'em new code won't fit
                int     21h
                mov     al,3            ; terminate with exit code of 3
                jmps    exit

itsok:          mov     si,ofs newboot  ; point to replacement code
                mov     cx,nbsize       ; load size of new boot code
                rep     movsb           ; and move it into place

                                        ; point si to location of value that is
                                        ; moved to si by the mov si instruction
                                        ; within our buffer
                mov     si,bx
                add     si,ofs bootsec + (movsi - newboot) + 3
                add     bx,isize+2      ; compute location of boot read code
                                        ;   relative to 7c00h
                add     [si],bx         ; revise mov si instruction
                
                mov     ax,301h         ; write 1 sector
                mov     bx,ofs bootsec  ; with new boot sector
                mov     cx,1            ; cylender 0, sector 1
                sub     dx,dx           ; of head 0, drive 0 (A drive)
                int     13h
                jnc     writeok         ; no error writing floppy, done
                mov     ah,9
                mov     dx,ofs nowrite  ; can't write to floppy boot sector
                int     21h
                mov     al,3            ; terminate with exit code of 4
                jmps    exit

writeok:        sub     al,al           ; set exit code to 0
exit:           mov     ah,4ch          ; return to DOS
                int     21h

bootsec:                                ; our boot sector buffer
code            ends
                end     entry
